@@ -17,54 +17,11 @@ Combine <- function(x, Data, Thinning=1)
      Acceptance.Rate <- round(sum(sapply(x, with, Acceptance.Rate) *
                sapply(x, with, Iterations)) /
                sum(sapply(x, with, Iterations)),5)
-     Adaptive <- round(mean(sapply(x, with, Adaptive)))
      Algorithm <- x[[1]]$Algorithm
-     if(is.matrix(x[[1]]$Covar)) {
-          if(sum(dim(x[[1]]$Covar) > 2)) {
-               Covar <- matrix(rowSums(sapply(x, with, Covar) *
-                    sapply(x, with, Iterations)) /
-                    sum(sapply(x, with, Iterations)),
-                    nrow(x[[1]]$Covar), ncol(x[[1]]$Covar))}
-          else {
-               Covar <- matrix(sum(sapply(x, with, Covar) *
-                    sapply(x, with, Iterations)) /
-                    sum(sapply(x, with, Iterations)),
-                    nrow(x[[1]]$Covar), ncol(x[[1]]$Covar))
-               }
-          }
-     else if(is.vector(x[[1]]$Covar)) {
-          Covar <- sum(sapply(x, with, Covar) *
-               sapply(x, with, Iterations)) /
-               sum(sapply(x, with, Iterations))
-          }
-     else {
-          Covar <- list(NULL)
-          for (i in 1:length(x[[1]]$Covar)) {
-               if(is.matrix(x[[1]]$Covar[[i]])) {
-                    if(sum(dim(x[[1]]$Covar[[i]]) > 2)) {
-                         Covar[[i]] <- matrix(rowSums(sapply(x, with, Covar[[i]]) *
-                              sapply(x, with, Iterations)) /
-                              sum(sapply(x, with, Iterations)),
-                              nrow(x[[1]]$Covar[[i]]), ncol(x[[1]]$Covar[[i]]))
-                         }
-                    else {
-                         Covar[[i]] <- matrix(sum(sapply(x, with, Covar[[i]]) *
-                              sapply(x, with, Iterations)) /
-                              sum(sapply(x, with, Iterations)),
-                              nrow(x[[1]]$Covar[[i]]), ncol(x[[1]]$Covar[[i]]))
-                         }
-                    }
-               else if(is.vector(x[[1]]$Covar[[i]])) {
-                    Covar[[i]] <- sum(sapply(x, with, Covar[[i]]) *
-                         sapply(x, with, Iterations)) /
-                         sum(sapply(x, with, Iterations))
-                    }
-               }
-          }
+     Covar <- x[[len.x]]$Covar
      Iterations <- sum(sapply(x, with, Iterations))
      Model <- x[[1]]$Model
-     Minutes <- max(sapply(x, with, Minutes))
-     Periodicity <- round(mean(sapply(x, with, Periodicity)))
+     Minutes <- sum(sapply(x, with, Minutes))
      Status <- round(mean(sapply(x, with, Status)))
      LIV <- x[[1]]$Parameters
      ### Combine
@@ -100,14 +57,16 @@ Combine <- function(x, Data, Thinning=1)
      rm(batch.list, HD, Ind, thinned2)
      ### Assess Thinning and ESS Size for all parameter samples
      cat("Assessing Thinning and ESS\n")
-     acf.temp <- matrix(1, trunc(10*log10(thinned.rows)), LIV)
+     acf.rows <- trunc(10*log10(thinned.rows))
+     acf.temp <- matrix(1, acf.rows, LIV)
      ESS1 <- Rec.Thin <- rep(1, LIV)
      for (j in 1:LIV) {
-          temp0 <- acf(thinned[,j], lag.max=nrow(acf.temp), plot=FALSE)
-          acf.temp[,j] <- abs(temp0$acf[2:{nrow(acf.temp)+1},,1])
+          temp0 <- acf(thinned[,j], lag.max=acf.rows, plot=FALSE)
+          if(length(temp0$acf[-1,1,1]) == acf.rows)
+               acf.temp[,j] <- abs(temp0$acf[-1,1,1])
           ESS1[j] <- ESS(thinned[,j])
           Rec.Thin[j] <- which(acf.temp[,j] <= 0.1)[1]*Thinning}
-     Rec.Thin <- ifelse(is.na(Rec.Thin), nrow(acf.temp), Rec.Thin)
+     Rec.Thin[which(is.na(Rec.Thin))] <- nrow(acf.temp)
      ### Assess ESS for all deviance and monitor samples
      ESS2 <- ESS(Dev)
      ESS3 <- ESS(Mon)
@@ -119,10 +78,10 @@ Combine <- function(x, Data, Thinning=1)
      ### Posterior Summary Table 1: All Thinned Samples
      cat("Creating Summaries\n")
      Num.Mon <- ncol(Mon)
-     Summ1 <- matrix(NA, LIV, 7, dimnames=list(Data$parm.names,
+     Summ1 <- matrix(NA, LIV, 7, dimnames=list(Data[["parm.names"]],
           c("Mean","SD","MCSE","ESS","LB","Median","UB")))
      Summ1[,1] <- colMeans(thinned)
-     Summ1[,2] <- apply(thinned, 2, sd)
+     Summ1[,2] <- sqrt(.colVars(thinned))
      Summ1[,3] <- 0
      Summ1[,4] <- ESS1
      Summ1[,5] <- apply(thinned, 2, quantile, c(0.025), na.rm=TRUE)
@@ -160,10 +119,10 @@ Combine <- function(x, Data, Thinning=1)
           Monitor[7] <- as.numeric(quantile(Mon[,j], probs=0.975,
                na.rm=TRUE))
           Summ1 <- rbind(Summ1, Monitor)
-          rownames(Summ1)[nrow(Summ1)] <- Data$mon.names[j]
+          rownames(Summ1)[nrow(Summ1)] <- Data[["mon.names"]][j]
           }
      ### Posterior Summary Table 2: Stationary Samples
-     Summ2 <- matrix(NA, LIV, 7, dimnames=list(Data$parm.names,
+     Summ2 <- matrix(NA, LIV, 7, dimnames=list(Data[["parm.names"]],
           c("Mean","SD","MCSE","ESS","LB","Median","UB")))
      if(Stat.at < thinned.rows) {
           thinned2 <- matrix(thinned[Stat.at:thinned.rows,],
@@ -173,7 +132,7 @@ Combine <- function(x, Data, Thinning=1)
           Mon2 <- matrix(Mon[Stat.at:thinned.rows,],
                thinned.rows-Stat.at+1, ncol(Mon))
           Summ2[,1] <- colMeans(thinned2)
-          Summ2[,2] <- apply(thinned2, 2, sd)
+          Summ2[,2] <- sqrt(.colVars(thinned2))
           Summ2[,3] <- 0
           Summ2[,4] <- ESS4
           Summ2[,5] <- apply(thinned2, 2, quantile, c(0.025), na.rm=TRUE)
@@ -216,38 +175,51 @@ Combine <- function(x, Data, Thinning=1)
                Monitor[7] <- as.numeric(quantile(Mon2[,j],
                     probs=0.975, na.rm=TRUE))
                Summ2 <- rbind(Summ2, Monitor)
-               rownames(Summ2)[nrow(Summ2)] <- Data$mon.names[j]}
+               rownames(Summ2)[nrow(Summ2)] <- Data[["mon.names"]][j]}
           }
      ### Column names to samples
-     if(identical(ncol(Mon), length(Data$mon.names)))
-          colnames(Mon) <- Data$mon.names
-     if(identical(ncol(thinned), length(Data$parm.names))) {
-          colnames(thinned) <- Data$parm.names}
+     if(identical(ncol(Mon), length(Data[["mon.names"]])))
+          colnames(Mon) <- Data[["mon.names"]]
+     if(identical(ncol(thinned), length(Data[["parm.names"]]))) {
+          colnames(thinned) <- Data[["parm.names"]]}
      ### Logarithm of the Marginal Likelihood
      LML <- list(LML=NA, VarCov=NA)
-     if(({Algorithm == "Affine-Invariant Ensemble Sampler"} |
-          {Algorithm == "Componentwise Hit-And-Run Metropolis"} |
-          {Algorithm == "Componentwise Slice"} |
-          {Algorithm == "Delayed Rejection Metropolis"} |
-          {Algorithm == "Hit-And-Run Metropolis"} | 
-          {Algorithm == "Hamiltonian Monte Carlo"} |
-          {Algorithm == "Independence Metropolis"} |
-          {Algorithm == "Metropolis-within-Gibbs"} |
-          {Algorithm == "No-U-Turn Sampler"} | 
-          {Algorithm == "Random-Walk Metropolis"} |
-          {Algorithm == "Reversible-Jump"} |
-          {Algorithm == "Sequential Metropolis-within-Gibbs"} |
-          {Algorithm == "Slice Sampler"} | 
-          {Algorithm == "Tempered Hamiltonian Monte Carlo"} | 
-          {Algorithm == "t-walk"}) & 
+     if(Algorithm %in% c("Adaptive Griddy-Gibbs",
+          "Affine-Invariant Ensemble Sampler",
+          "Automated Factor Slice Sampler",
+          "Componentwise Hit-And-Run Metropolis",
+          "Delayed Rejection Metropolis",
+          "Elliptical Slice Sampler",
+          "Gibbs Sampler",
+          "Griddy-Gibbs",
+          "Hamiltonian Monte Carlo",
+          "Hit-And-Run Metropolis",
+          "Independence Metropolis",
+          "Metropolis-Adjusted Langevin Algorithm",
+          "Metropolis-Coupled Markov Chain Monte Carlo",
+          "Metropolis-within-Gibbs",
+          "Multiple-Try Metropolis",
+          "No-U-Turn Sampler",
+          "Oblique Hyperrectangle Slice Sampler",
+          "Preconditioned Crank-Nicolson",
+          "Random Dive Metropolis-Hastings",
+          "Random-Walk Metropolis",
+          "Reflective Slice Sampler",
+          "Refractive Sampler",
+          "Reversible-Jump",
+          "Sequential Metropolis-within-Gibbs",
+          "Slice Sampler",
+          "Stochastic Gradient Langevin Dynamics",
+          "Tempered Hamiltonian Monte Carlo",
+          "t-walk",
+          "Univariate Eigenvector Slice Sampler") &
           {Stat.at < thinned.rows}) {
           cat("Estimating Log of the Marginal Likelihood\n")
-          LML <- LML(theta=thinned2,
-               LL=as.vector(Dev2)*(-1/2), method="NSIS")}
+          LML <- LML(theta=thinned2, LL=as.vector(Dev2)*(-1/2),
+               method="NSIS")}
      ### Compile Output
      cat("Creating Output\n")
      LaplacesDemon.out <- list(Acceptance.Rate=Acceptance.Rate,
-          Adaptive=Adaptive,
           Algorithm=Algorithm,
           Call=x[[1]]$Call,
           Covar=Covar,
@@ -262,7 +234,6 @@ Combine <- function(x, Data, Thinning=1)
                mean(as.vector(Dev2)) + 
                var(as.vector(Dev2))/2)}
                else rep(NA,3),
-          DR=x[[1]]$DR,
           Initial.Values=x[[len.x]]$Initial.Values,
           Iterations=Iterations,
           LML=LML[[1]],
@@ -270,7 +241,6 @@ Combine <- function(x, Data, Thinning=1)
           Model=Model,
           Monitor=Mon,
           Parameters=LIV,
-          Periodicity=Periodicity,
           Posterior1=thinned,
           Posterior2=if(Stat.at < thinned.rows) {
                thinned[Stat.at:thinned.rows,]}
@@ -278,6 +248,7 @@ Combine <- function(x, Data, Thinning=1)
           Rec.BurnIn.Thinned=BurnIn,
           Rec.BurnIn.UnThinned=BurnIn*Thinning,
           Rec.Thinning=min(1000, max(Rec.Thin)),
+          Specs=x[[1]]$Specs,
           Status=Status,
           Summary1=Summ1,
           Summary2=Summ2,
